/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2015 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */

define([
    "jquery",
    "base1/cockpit",
    "docker/react",
    "docker/cockpit-components-dialog",
    "data!docker/cockpit-atomic-storage"
], function($, cockpit, React, dialog_view, cockpit_atomic_storage) {
    var _ = cockpit.gettext;
    var C_ = cockpit.gettext;

    function init_model(callback) {
        var process;

        var self = {
            error: null,
            driver: "",
            pool_devices: [ ],
            extra_devices: [ ],
            total: 0,
            used: 0
        };

        function cmp_drive(a, b) {
            return a.sort_index - b.sort_index;
        }

        function update() {
            if (!cockpit.hidden && !process) {
                process = cockpit.spawn([ "python", "-c", cockpit_atomic_storage, "monitor" ],
                                        { err: "ignore",
                                          superuser: true }).
                                  stream(function (data) {
                                      // XXX - find the newlines here
                                      var info = JSON.parse(data);
                                      self.loopback = info.loopback;
                                      self.pool_devices = info.pool_devices.sort(cmp_drive);
                                      self.extra_devices = info.extra_devices.sort(cmp_drive);
                                      self.total = info.total;
                                      self.used = info.used;
                                      self.error = null;
                                      if (!info.can_manage)
                                          self.error = "unsupported";
                                      $(self).triggerHandler("changed");
                                  }).
                                  fail(function (error) {
                                      if (error != "closed") {
                                          console.warn(error);
                                          self.error = error.problem || "broken";
                                          $(self).triggerHandler("changed");
                                      }
                                  });
            } else if (cockpit.hidden && process) {
                process.close("closed");
                process = null;
            }
        }

        $(cockpit).on("visibilitychange", update);
        update();

        return self;
    }

    var storage_model = null;

    function get_storage_model() {
        if (!storage_model)
            storage_model = init_model();
        return storage_model;
    }

    /* A list of unused drives ready to be added to the Docker storage pool.
     *
     * model: The model as returned by get_storage_model.
     *
     * callback: Called as callback(drives, needs_reset) when
     *           the "Add" button is clicked.
     *
     */
    var DriveBox = React.createClass({displayName: "DriveBox",
        getInitialState: function () {
            return {
                drives: this.props.model.extra_devices,
                checked: { }
            };
        },
        onModelChanged: function () {
            this.setState({ drives: this.props.model.extra_devices });
        },
        componentDidMount: function () {
            $(this.props.model).on("changed", this.onModelChanged);
            this.onModelChanged();
        },
        componentWillUnmount: function () {
            $(this.props.model).off("changed", this.onModelChanged);
        },

        driveChecked: function (drive) {
            return !!this.state.checked[drive.path];
        },
        toggleDrive: function (drive) {
            this.state.checked[drive.path] = !this.state.checked[drive.path];
            this.setState({ checked: this.state.checked });
        },

        onButtonClicked: function () {
            var self = this;
            if (self.props.callback) {
                var drives = [ ];
                for (var d in self.state.checked) {
                    if (self.state.checked[d]) {
                        for (var i = 0; i < self.state.drives.length; i++) {
                            if (self.state.drives[i].path == d) {
                                drives.push(self.state.drives[i]);
                            }
                        }
                    }
                }
                self.setState({ checked: { } });
                self.props.callback(drives, self.props.model.loopback);
            }
        },

        render: function() {
            var self = this;
            var i, d;

            var button_enabled = false;
            var drive_rows = [ ];
            var drive_paths = { };

            function drive_class_desc(cl) {
                switch (cl) {
                    case "sdd": return _("Solid-State Disk");
                    case "hdd": return _("Hard Disk");
                    default:    return _("Drive");
                }
            }

            for (i = 0; i < self.state.drives.length; i++) {
                var drive = self.state.drives[i];

                if (self.state.checked[drive.path])
                    button_enabled = true;

                drive_paths[drive.path] = true;

                drive_rows.push(
                    React.createElement("tr", {onClick: self.toggleDrive.bind(self, drive)}, 
                        React.createElement("td", null, React.createElement("input", {type: "checkbox", 
                                   checked: self.driveChecked(drive)})
                        ), 
                        React.createElement("td", null, React.createElement("img", {src: "images/drive-harddisk-symbolic.svg"})), 
                        React.createElement("td", null, 
                            React.createElement("div", null, drive.name), 
                            React.createElement("div", null, cockpit.format_bytes(drive.size), " ", drive_class_desc(drive.class))
                        )
                    ));
            }

            var drive_list;

            if (drive_rows.length > 0) {
                drive_list = (
                    React.createElement("div", null, 
                        React.createElement("h4", null, _("Local Disks")), 
                        React.createElement("table", null, 
                             drive_rows 
                        )
                    ));
            } else {
                drive_list = React.createElement("span", null, _("No additional local storage found."));
            }

            return (
                React.createElement("div", {className: "drives-panel"}, 
                    React.createElement("div", {className: "drives-panel-body"}, 
                         drive_list 
                    ), 
                    React.createElement("div", {className: "drives-panel-footer"}, 
                        React.createElement("button", {className: "btn btn-primary", 
                                disabled:  !button_enabled, 
                                onClick:  self.onButtonClicked}, "Add Storage")
                    )
                )
            );
        }
    });

    /* A list of drives in the Docker storage pool.
     *
     * model: The model as returned by get_storage_model.
     */
    var PoolBox = React.createClass({displayName: "PoolBox",
        getInitialState: function () {
            return {
                drives: [ ]
            };
        },
        onModelChanged: function () {
            this.setState({ drives: this.props.model.pool_devices });
        },
        componentDidMount: function () {
            $(this.props.model).on("changed", this.onModelChanged);
            this.onModelChanged();
        },
        componentWillUnmount: function () {
            $(this.props.model).off("changed", this.onModelChanged);
        },

        render: function() {
            var self = this;

            function render_drive_rows() {
                return self.state.drives.map(function (drive) {
                    return (
                        React.createElement("tr", null, 
                            React.createElement("td", null, cockpit.format_bytes(drive.size)), 
                            React.createElement("td", null, React.createElement("img", {src: "images/drive-harddisk-symbolic.svg"})), 
                            React.createElement("td", null, drive.name, drive.shared? _(" (shared with the OS)"):"")
                        ));
                });
            }

            return (
                React.createElement("table", {className: "drive-list"}, 
                    render_drive_rows()
                ));
        }
    });

    /* A overview of the Docker Storage pool size and how is used.
     *
     * model: The model as returned by get_storage_model.
     *
     * small: If true, a small version is rendered
     *        with a link to the setup page.
     */
    var OverviewBox = React.createClass({displayName: "OverviewBox",
        getInitialState: function () {
            return { total: 0, used: 0 };
        },
        onModelChanged: function () {
            this.setState({ error: this.props.model.error,
                            total: this.props.model.total,
                            used: this.props.model.used });
        },
        componentDidMount: function () {
            $(this.props.model).on("changed", this.onModelChanged);
            this.onModelChanged();
        },
        componentWillUnmount: function () {
            $(this.props.model).off("changed", this.onModelChanged);
        },

        render: function() {
            var self = this;

            if (!self.state.total) {
                return (React.createElement("div", null, _("Information about the Docker storage pool is not available.")));
            }

            var total_fmt = cockpit.format_bytes(self.state.total, undefined, true);
            var used_fmt = cockpit.format_bytes(self.state.used, total_fmt[1], true);
            var free_fmt = cockpit.format_bytes(self.state.total - self.state.used, undefined, true);

            var used_perc = (self.state.used / self.state.total) * 100 + "%";

            if (self.props.small) {
                return (
                    React.createElement("div", null, 
                        React.createElement("div", null, 
                            React.createElement("div", {className: "used-total"}, used_fmt[0], " / ", total_fmt[0], " ", total_fmt[1]), 
                            React.createElement("div", null, 
                                React.createElement("span", {className: "free-text"}, free_fmt[0], " "), 
                                React.createElement("span", {className: "free-unit"}, free_fmt[1], " "), React.createElement("span", null, _("Free"))
                            )
                        ), 
                        React.createElement("div", {className: "progress"}, 
                            React.createElement("div", {className: "progress-bar", style: {width: used_perc}}
                            )
                        ), 
                        self.state.error? "" : React.createElement("a", {translatable: "yes", href: "#/storage"}, "configure storage...")
                    ));
            } else {
                return (
                    React.createElement("div", null, 
                        React.createElement("div", null, 
                            React.createElement("div", {className: "used-total"}, 
                                React.createElement("table", null, 
                                    React.createElement("tr", null, React.createElement("td", null, _("Used")), React.createElement("td", null, used_fmt[0], " ", used_fmt[1])), 
                                    React.createElement("tr", null, React.createElement("td", null, _("Total")), React.createElement("td", null, total_fmt[0], " ", total_fmt[1]))
                                )
                            ), 
                            React.createElement("div", null, 
                                React.createElement("span", {className: "free-text"}, free_fmt[0]), 
                                React.createElement("div", {className: "free-unit"}, 
                                    free_fmt[1], React.createElement("br", null), _("Free")
                                )
                            )
                        ), 
                        React.createElement("div", {className: "progress"}, 
                            React.createElement("div", {className: "progress-bar c2", style: {width: used_perc}}
                            )
                        )
                    ));
            }
        }
    });

    function add_storage(client, drives, loopback) {
        function render_drive_rows() {
            return drives.map(function (drive) {
                return (
                    React.createElement("tr", null, 
                        React.createElement("td", null, cockpit.format_bytes(drive.size)), 
                        React.createElement("td", null, React.createElement("img", {src: "images/drive-harddisk-symbolic.svg"})), 
                        React.createElement("td", null, drive.name)
                    ));
            });
        }

        var reset_warning = null;
        var storage_action = "add";
        var docker_will_be_stopped = false;
        var action_caption = _("Reformat and add disks");

        if (loopback) {
            reset_warning = (
                React.createElement("div", {className: "alert alert-danger"}, 
                    React.createElement("span", {className: "fa fa-exclamation-triangle"}), 
                    React.createElement("span", {className: "alert-message"}, 
                        _("The storage pool will be reset to optimize its layout.  All containers will be erased.")
                    )
                ));
            storage_action = "reset-and-add";
            docker_will_be_stopped = true;
            action_caption = _("Erase containers, reformat disks, and add them");
        }

        dialog_view.show_modal_dialog({ 'title': _("Add Additional Storage"),
                                        'body': (
                                            React.createElement("div", {className: "modal-body"}, 
                                                React.createElement("p", null, _("All data on selected disks will be erased and disks will be added to the storage pool.")), 
                                                React.createElement("table", {className: "drive-list"}, 
                                                     render_drive_rows() 
                                                ), 
                                                 reset_warning 
                                            )),
                                      },
                                      { 'actions': [ { 'caption': action_caption,
                                                       'clicked': add_drives,
                                                       'style': "danger" } ]
                                      });

        function add_drives() {
            var dfd = $.Deferred();
            var devs = drives.map(function (d) { return d.path; });
            if (docker_will_be_stopped)
                client.close();
            var process = cockpit.spawn([ "python", "-c", cockpit_atomic_storage, storage_action ].concat(devs),
                                         { 'err': 'out',
                                           'superuser': true }).
                                  done(function (data) {
                                      if (docker_will_be_stopped) {
                                          client.connect().done(function () {
                                              dfd.resolve();
                                          });
                                      } else {
                                          dfd.resolve();
                                      }
                                  }).
                                  fail(function (error, data) {
                                      if (docker_will_be_stopped)
                                          client.connect();
                                      if (error.problem == "cancelled") {
                                          dfd.resolve();
                                          return;
                                      }
                                      dfd.reject(
                                          React.createElement("div", null, 
                                              React.createElement("span", null, _("Could not add all disks")), 
                                              React.createElement("pre", null, data)
                                          ));
                                  });
            var promise = dfd.promise();
            promise.cancel = function () {
                process.close("cancelled");
            };
            return promise;
        }
    }

    function reset_storage(client, service) {
        dialog_view.show_modal_dialog({ 'title': _("Reset Storage Pool"),
                                        'body': (
                                            React.createElement("div", {className: "modal-body"}, 
                                                React.createElement("p", null, _("Resetting the storage pool will erase all containers and release disks in the pool."))
                                            )),
                                      },
                                      { 'actions': [ { 'caption': _("Erase containers and reset storage pool"),
                                                       'clicked': reset,
                                                       'style': "danger" } ]
                                      });
        function reset() {
            var dfd = $.Deferred();
            client.close();
            var process = cockpit.spawn([ "python", "-c", cockpit_atomic_storage, "reset-and-reduce" ],
                                        { 'err': 'out',
                                          'superuser': true }).
                                  done(function (data) {
                                      client.connect().done(function () {
                                          dfd.resolve();
                                      });
                                  }).
                                  fail(function (error, data) {
                                      client.connect();
                                      if (error.problem == "cancelled") {
                                          dfd.resolve();
                                          return;
                                      }
                                      dfd.reject(
                                          React.createElement("div", null, 
                                              React.createElement("span", null, _("Could not reset the storage pool")), 
                                              React.createElement("pre", null, data)
                                          ));
                                  });
            var promise = dfd.promise();
            promise.cancel = function () {
                process.close("cancelled");
            };
            return promise;
        }
    }

    function init_storage(client, service) {
        $('#storage .breadcrumb a').on("click", function() {
            cockpit.location.go('/');
        });

        $('#storage-reset').on('click', function () { reset_storage(client, service); });

        function add_callback(drives, driver) {
            add_storage(client, drives, driver);
        }

        var model = get_storage_model();

        React.render(React.createElement(DriveBox, {model: model, callback: add_callback}),
                     $("#storage-drives")[0]);
        React.render(React.createElement(PoolBox, {model: model}),
                     $("#storage-pool")[0]);
        React.render(React.createElement(OverviewBox, {model: model}),
                     $("#storage-overview")[0]);

        function update_curtain() {
            if (model.error) {
                if (model.error == "access-denied")
                    $('#storage-unsupported-message').text(
                        _("You don't have permission to manage the Docker storage pool."));
                else
                    $('#storage-unsupported-message').text(
                        _("The Docker storage pool can not be managed on this system."));
                $("#storage-unsupported").show();
                $("#storage-details").hide();
            } else {
                $("#storage-unsupported").hide();
                $("#storage-details").show();
            }
        }

        $(model).on("changed", update_curtain);
        update_curtain();

        function hide() {
            $('#storage').hide();
        }

        function show(id) {
            $('#storage').show();
        }

        return {
            show: show,
            hide: hide
        };
    }

    return {
        get_storage_model: get_storage_model,
        OverviewBox: OverviewBox,

        init: init_storage
    };
});
